@@ -22,7 +22,7 @@ imp_orderedcat_single <- function(y_imp, X_imp){
   missind <- is.na(y_imp)
 
   #starting model
-  ph <- sample_imp(y_imp)
+  ph <- sample_imp(y_imp)[, 1]
   tmp_0_all <- data.frame(target = ph)
   xnames_0 <- paste("X", 1:ncol(X_imp_stand), sep = "")
   tmp_0_all[xnames_0] <- X_imp_stand
@@ -58,11 +58,11 @@ imp_orderedcat_single <- function(y_imp, X_imp){
 
 
   #Initialising the returning vector
-  y_ret <- as.matrix(y_imp, ncol = 1)
+  y_ret <- data.frame(y_imp)
 
   y_ret[missind, 1] <- everything$imp[[1]][, 1]
 
-  return(data.frame(y_imp = factor(y_ret)))
+  return(y_ret)
 }
 
 
