@@ -2,7 +2,7 @@
 #'
 #' This functions imputes interval data variables. Those are variables,
 #' that consists of a lower and upper (numeric) boundary. Technically
-#' those boundaries are cointained in a string, separated by a semi colon.
+#' those boundaries are contained in a string, separated by a semi colon.
 #' E.g. if a person reports there income to be something between 3000 and 4000 dollars,
 #' its value in the interval covariate would be \code{"3000;4000"}.
 #' Left (resp. right) censored data can be denoted by \code{"-Inf;x"} (resp. \code{"x;Inf"}),
@@ -24,49 +24,59 @@ imp_interval <- function(y_imp, X_imp){
 
   missind <- is.na(y_imp)
 
-  # blob has to be numeric, so it must only consists of precise observations
+  # has to be numeric, so it must only consists of precise observations
   decomposed <- decompose_interval(interval = y_imp)
 
-  if(any(decomposed$lower > decomposed$upper, na.rm = TRUE)){
+  if(any(decomposed$lower_general > decomposed$upper_general, na.rm = TRUE)){
     stop("in your interval covariate, some values in the lower bound exceed the upper bound.")
   }
 
-  y_precise_template <- decomposed$precise
+  y_precise_template <- sample_imp(center.interval(y_imp, inf2NA = TRUE))[, 1]
   n <- length(y_precise_template)
+
+  y_mean <- mean(y_precise_template)
+  y_sd <- stats::sd(y_precise_template)
+  #add 1 to avoid an exactly zero intercept when modeling y_stand ~ x_stand
+  # If both, X and Y, are standardized, the intercept
+  #will be exactly 0 and thus not significantly different from 0.
+  #So in order to avoid this variable to be removed later in the code, we add +1.
+  decomposed_stand <- (decomposed - y_mean)/y_sd + 1
+  y_precise_template_stand <- (y_precise_template - y_mean)/y_sd + 1
   #if there are imprecise values only...
-  if(all(is.na(y_precise_template))){
+  #if(all(is.na(y_precise_template))){
     #... the template will be set up with a draw from between the borders
-    low_sample <- sample_imp(decomposed$lower)
-    up_sample <- sample_imp(decomposed$upper)
+    low_sample <- decomposed_stand$lower_general
+    up_sample <- decomposed_stand$upper_general
+
     y_precise_template <- msm::rtnorm(n = n, lower = low_sample,
                                       upper = up_sample,
-                                      mean = 0,
+                                      mean = y_precise_template_stand,
                                       sd = 1)
-  }
+  #}
 
-  ph <- sample_imp(y_precise_template)
-  tmp_1 <- data.frame(y = ph)
+  ph_stand <- y_precise_template
+  tmp_1 <- data.frame(target = ph_stand)
 
   # run a linear model to get the suitable model.matrix for imputation of the NAs
-  lmstart <- stats::lm(ph ~ 0 + . , data = X_imp_stand)
+  lmstart <- stats::lm(ph_stand ~ 0 + . , data = X_imp_stand)
   X_model_matrix_1 <- stats::model.matrix(lmstart)
   xnames_1 <- paste("X", 1:ncol(X_model_matrix_1), sep = "")
   tmp_1[, xnames_1] <- X_model_matrix_1
 
 
-  fixformula_1 <- stats::formula(paste("y ~ 0 +", paste(xnames_1, collapse = "+"), sep = ""))
+  fixformula_1 <- stats::formula(paste("target ~ 0 +", paste(xnames_1, collapse = "+"), sep = ""))
 
 
   reg_1 <- stats::lm(fixformula_1, data = tmp_1)
 
   # remove variables with an NA coefficient
 
-  tmp_2 <- data.frame(y = ph)
+  tmp_2 <- data.frame(target = ph_stand)
 
   xnames_2 <- xnames_1[!is.na(stats::coefficients(reg_1))]
   tmp_2[, xnames_2] <- X_model_matrix_1[, !is.na(stats::coefficients(reg_1)), drop = FALSE]
 
-  fixformula_2 <- stats::formula(paste("y ~ 0 +", paste(xnames_2, collapse = "+"), sep = ""))
+  fixformula_2 <- stats::formula(paste("target ~ 0 +", paste(xnames_2, collapse = "+"), sep = ""))
   reg_2 <- stats::lm(fixformula_2, data = tmp_2)
   X_model_matrix_2 <- stats::model.matrix(reg_2)
 
@@ -74,14 +84,19 @@ imp_interval <- function(y_imp, X_imp){
   coef.std <- sqrt(diag(stats::vcov(reg_2)))
 
 
-  includes_unimportants <- any(coef.std > max.se) | any(stats::coef(reg_2) < 1e-03)
+  includes_unimportants <- any(coef.std > max.se) | any(abs(stats::coef(reg_2)) < 1e-03)
   counter <- 0
   while(includes_unimportants & counter <= ncol(X_model_matrix_2)){
     counter <- counter + 1
 
     X_model_matrix_2 <- as.data.frame(X_model_matrix_2[,
                                       coef.std <= max.se & stats::coef(reg_2) >= 1e-03, drop = FALSE])
-    reg_2 <- stats::lm(ph ~ 0 + . , data = X_model_matrix_2)
+    if(ncol(X_model_matrix_2) == 0){
+      reg_2 <- stats::lm(ph_stand[, 1] ~ 1)
+    }else{
+      reg_2 <- stats::lm(ph_stand[, 1] ~ 0 + . , data = X_model_matrix_2)
+    }
+
     #remove regression parameters which have a very high standard error
     max.se <- abs(stats::coef(reg_2) * 3)
     coef.std <- sqrt(diag(stats::vcov(reg_2)))
@@ -99,7 +114,7 @@ imp_interval <- function(y_imp, X_imp){
   #####maximum likelihood estimation using starting values
   ####estimation of the parameters
 
-  lmstart2 <- stats::lm(ph ~ 0 + ., data = MM_1) # it might be more practical to run the model
+  lmstart2 <- stats::lm(ph_stand~ 0 + ., data = MM_1) # it might be more practical to run the model
   #only based on the observed data, but this could cause some covariates in betastart2 to be dropped
   betastart2 <- as.vector(lmstart2$coef)
   sigmastart2 <- stats::sigma(lmstart2)
@@ -131,12 +146,14 @@ imp_interval <- function(y_imp, X_imp){
   keep <- (y_precise_template >= quants[1] & y_precise_template <= quants[2]) |
     is.na(y_precise_template)
 
+
   negloglik2_generated <- function_generator(para = starting_values,
                                              X = MM_1[keep, , drop = FALSE],
-                                             lower = decomposed$lower[keep],
-                                             upper = decomposed$upper[keep])
+                                             lower = decomposed_stand$lower_imprecise[keep],
+                                             upper = decomposed_stand$upper_imprecise[keep])
 
-  m2 <- stats::optim(par = starting_values, negloglik2_generated, method = "BFGS",
+  m2 <- stats::optim(par = starting_values, negloglik2_generated,
+                     method = "BFGS",#alternative: "Nelder-Mead"
                      control = list(maxit = 10000), hessian = TRUE)
 
 
@@ -148,8 +165,35 @@ imp_interval <- function(y_imp, X_imp){
   # nearPD(hess)$mat
   # isSymmetric(Sigma_ml2)
 
-  Sigma_ml2 <- diag(diag(solve(Matrix::nearPD(hess)$mat)))
-  diag(Sigma_ml2) <- pmax(diag(Sigma_ml2), 1e-5)
+
+  Sigma_ml2 <- tryCatch(
+    {
+
+      Sigma_ml2 <- solve(hess)
+
+    },
+    error = function(cond) {
+      cat("Hessian matrix couldn't be inverted (in the imputation function of the rounded continuous variable).
+              Still, you should get a result, but which needs special attention.\n")
+      cat("Here's the original error message:\n")
+      cat(as.character(cond))
+
+      Sigma_ml2 <- diag(diag(solve(Matrix::nearPD(hess)$mat)))
+      diag(Sigma_ml2) <- pmax(diag(Sigma_ml2), 1e-5)
+
+    },
+    warning = function(cond) {
+      cat("There seems to be a problem with the Hessian matrix in the imputation of the rounded continuous variable.\n")
+      cat("Here's the original warning message:\n")
+      cat(as.character(cond))
+
+      Sigma_ml2 <- solve(hess)
+
+    },
+    finally = {
+    }
+  )
+
   # make sure, that the main diagonal elements are non-zero
 
   ###set starting values equal to the observed income
@@ -179,15 +223,9 @@ imp_interval <- function(y_imp, X_imp){
   #for this purpose we have to replace the lower and upper bounds
   # of those observations with an NA in y_imp by -Inf and Inf
 
-  expanded_lower <- decomposed$lower
-  expanded_lower[!is.na(decomposed$precise)] <-
-      decomposed$precise[!is.na(decomposed$precise)]
-  expanded_lower[is.na(expanded_lower)] <- -Inf
+  expanded_lower <- decomposed_stand$lower_general
 
-  expanded_upper <- decomposed$upper
-  expanded_upper[!is.na(decomposed$precise)] <-
-      decomposed$precise[!is.na(decomposed$precise)]
-  expanded_upper[is.na(expanded_upper)] <- Inf
+  expanded_upper <- decomposed_stand$upper_general
 
 
   #draw values from the truncated normal distributions
@@ -203,8 +241,8 @@ imp_interval <- function(y_imp, X_imp){
                                    mean = as.matrix(MM_1) %*% beta_hat,
                                    sd = sqrt(Sigma))
 
-  y_ret <- tnorm_draws
-
+  #undo the standardization
+  y_ret <- (tnorm_draws - 1) * y_sd + y_mean
 
   return(data.frame(y_imp = y_ret))
 }
